# Copyright 2017 Cumulus Networks, Inc.  All Rights reserved.
#
# This software is subject to the Cumulus Networks End User License Agreement
# available at the following locations:
#
# Internet: https://cumulusnetworks.com/downloads/eula/latest/view/
# Cumulus Linux systems: /usr/share/cumulus/EULA.txt

from __future__ import absolute_import

import ujson as json

from netq_lib.orm.redisdb.models import (
    Temp,
    Fan,
    PSU)
from netq_lib.common.enums import ServiceStatus
from netq_lib.common.utils import save_fullstate

local_sensor_dict = dict()
FAIL_TEMP = -273
FAIL_FAN = -1


def process_sensors(output, svc_failed=False, **kwargs):


    fullstate = kwargs.get('fullstate')

    if svc_failed:
        return ServiceStatus.FAIL
    else:
        try:
            s_list = json.loads(output)
        except ValueError:
            return ServiceStatus.FAIL

    temp_arr = []
    psu_arr = []
    fan_arr = []
    for s in s_list:
        sensor = None
        name = s['name']
        if not name:
            continue
        if s['type'] == 'temp':
            sensor = Temp()
            sensor.s_input = (float(s['input']) if s.get('input') is not None
                              else float(FAIL_TEMP))
            sensor.s_crit = int(s['crit']) if s.get('crit') is not None else FAIL_TEMP
            sensor.s_max = int(s['max']) if s.get('max') is not None else FAIL_TEMP
            sensor.s_min = int(s['min']) if s.get('min') is not None else FAIL_TEMP
            sensor.s_lcrit = int(s['lcrit']) if s.get('lcrit') is not None else FAIL_TEMP
            sensor.s_desc = s['description'].encode('ascii', 'ignore').lower()
            # Naga: Temporarily comment below lines to have stream all the time
            #if local_sensor_dict.get(name):
            #    prev_val, prev_st = local_sensor_dict[name]
            #    if ((cur_st == prev_st) and
            #            (abs(sensor.s_input - prev_val) < 2.5)):
            #        continue
            #local_sensor_dict[name] = (sensor.s_input, cur_st)
        elif s['type'] == 'fan':
            sensor = Fan()
            sensor.s_input = int(s['input']) if s.get('input') is not None else FAIL_FAN
            sensor.s_max = int(s['max']) if s.get('max') is not None else FAIL_FAN
            sensor.s_min = int(s['min']) if s.get('min') is not None else FAIL_FAN
            sensor.s_desc = s['description'].encode('ascii', 'ignore').lower()
            # Naga: Temporarily comment below lines to have stream all the time
            #if local_sensor_dict.get(name):
            #    prev_val, prev_st = local_sensor_dict[name]
            #    if ((cur_st == prev_st) and
            #            abs(sensor.s_input - prev_val) < s_max_val):
            #        continue
            #local_sensor_dict[name] = (sensor.s_input, cur_st)
        elif s['type'] == 'power':
            sensor = PSU()

        if sensor is not None:
            sensor.s_name = s['name'].encode('ascii', 'ignore').lower()
            if not s['msg']:
                sensor.s_msg = ''
            else:
                msg = s['msg'].encode('ascii', 'ignore')

                msgs = msg.split('\n')
                m = []
                for msg in msgs:
                    m.append(msg.split(':', 1)[1])
                sensor.s_msg = ','.join(m)
            sensor.s_state = s['state'].encode('ascii', 'ignore').lower()
            sensor.s_prev_state = s['prev_state'].encode('ascii',
                                                         'ignore').lower()

            if fullstate:
                if sensor.__class__.__name__ == "Temp":
                    temp_arr.append(sensor)
                elif sensor.__class__.__name__ == "Fan":
                    fan_arr.append(sensor)
                else:
                    psu_arr.append(sensor)
            else:
                sensor.save()
    if fullstate:
        save_fullstate(temp_arr, "Temp", False)
        save_fullstate(fan_arr, "Fan", False)
        save_fullstate(psu_arr, "PSU", False)

    return ServiceStatus.OK
